@@ -84,10 +84,10 @@ prep_stmt::operator =(const prep_stmt& x)
 
 struct database : public database_i, private noncopyable {
   database(const config& c);
-  virtual ~database();
-  virtual dbcontext_ptr create_context(bool for_write) volatile;
-  virtual void stop() volatile;
-  virtual const config& get_conf() const volatile;
+  ~database() override;
+  dbcontext_ptr create_context(bool for_write) volatile override;
+  void stop() volatile override;
+  const config& get_conf() const volatile override;
  public:
   int child_running;
  private:
@@ -128,21 +128,21 @@ struct expr_user_lock : private noncopyable {
 
 struct dbcontext : public dbcontext_i, private noncopyable {
   dbcontext(volatile database *d, bool for_write);
-  virtual ~dbcontext();
-  virtual void init_thread(const void *stack_botton,
-    volatile int& shutdown_flag);
-  virtual void term_thread();
-  virtual bool check_alive();
-  virtual void lock_tables_if();
-  virtual void unlock_tables_if();
-  virtual bool get_commit_error();
-  virtual void clear_error();
-  virtual void close_tables_if();
-  virtual void table_addref(size_t tbl_id);
-  virtual void table_release(size_t tbl_id);
-  virtual void cmd_open(dbcallback_i& cb, const cmd_open_args& args);
-  virtual void cmd_exec(dbcallback_i& cb, const cmd_exec_args& args);
-  virtual void set_statistics(size_t num_conns, size_t num_active);
+  ~dbcontext() override;
+  void init_thread(const void *stack_botton,
+    volatile int& shutdown_flag) override;
+  void term_thread() override;
+  bool check_alive() override;
+  void lock_tables_if() override;
+  void unlock_tables_if() override;
+  bool get_commit_error() override;
+  void clear_error() override;
+  void close_tables_if() override;
+  void table_addref(size_t tbl_id) override;
+  void table_release(size_t tbl_id) override;
+  void cmd_open(dbcallback_i& cb, const cmd_open_args& args) override;
+  void cmd_exec(dbcallback_i& cb, const cmd_exec_args& args) override;
+  void set_statistics(size_t num_conns, size_t num_active) override;
  private:
   int set_thread_message(const char *fmt, ...)
     __attribute__((format (printf, 2, 3)));
@@ -294,11 +294,11 @@ dbcontext::init_thread(const void *stack_bottom, volatile int& shutdown_flag)
       thd->options |= OPTION_BIN_LOG;
       #endif
       safeFree((char*) thd->db.str);
-      thd->db.str= my_strdup("handlersocket", MYF(0));
+      thd->db.str= my_strdup(PSI_NOT_INSTRUMENTED, "handlersocket", MYF(0));
       thd->db.length= sizeof("handlersocket")-1;
     }
     thd->variables.option_bits |= OPTION_TABLE_LOCK;
-    my_pthread_setspecific_ptr(THR_THD, thd);
+    set_current_thd(thd);
     DBG_THR(fprintf(stderr, "HNDSOCK x0 %p\n", thd));
   }
   {
@@ -335,7 +335,7 @@ dbcontext::term_thread()
 {
   DBG_THR(fprintf(stderr, "HNDSOCK thread end %p\n", thd));
   close_tables_if();
-  my_pthread_setspecific_ptr(THR_THD, 0);
+  set_current_thd(nullptr);
   {
     delete thd;
     thd = 0;
@@ -1000,7 +1000,7 @@ dbcontext::cmd_open(dbcallback_i& cb, const cmd_open_args& arg)
     LEX_CSTRING db_name=  { arg.dbn, strlen(arg.dbn) };
     LEX_CSTRING tbl_name= { arg.tbl, strlen(arg.tbl) };
     tables.init_one_table(&db_name, &tbl_name, 0, lock_type);
-    tables.mdl_request.init(MDL_key::TABLE, arg.dbn, arg.tbl,
+    MDL_REQUEST_INIT(&tables.mdl_request, MDL_key::TABLE, arg.dbn, arg.tbl,
       for_write_flag ? MDL_SHARED_WRITE : MDL_SHARED_READ, MDL_TRANSACTION);
     Open_table_context ot_act(thd, 0);
     if (!open_table(thd, &tables, &ot_act)) {
