@@ -20,6 +20,9 @@
                                              // mysql_exchange_partition
 #include "sql_statistics.h"                  // delete_statistics_for_column
 #include "sql_alter.h"
+#include "rpl_mi.h"
+#include "slave.h"
+#include "debug_sync.h"
 #include "wsrep_mysqld.h"
 
 Alter_info::Alter_info(const Alter_info &rhs, MEM_ROOT *mem_root)
@@ -176,8 +179,8 @@ bool Alter_info::supports_algorithm(THD *thd,
 }
 
 
-bool Alter_info::supports_lock(THD *thd,
-                               const Alter_inplace_info *ha_alter_info)
+bool Alter_info::supports_lock(THD *thd, bool online,
+                               Alter_inplace_info *ha_alter_info)
 {
   switch (ha_alter_info->inplace_supported) {
   case HA_ALTER_INPLACE_EXCLUSIVE_LOCK:
@@ -206,8 +209,13 @@ bool Alter_info::supports_lock(THD *thd,
   case HA_ALTER_INPLACE_SHARED_LOCK:
     if (requested_lock == Alter_info::ALTER_TABLE_LOCK_NONE)
     {
-      ha_alter_info->report_unsupported_error("LOCK=NONE", "LOCK=SHARED");
-      return true;
+      if (online)
+        ha_alter_info->inplace_supported= HA_ALTER_INPLACE_NOT_SUPPORTED;
+      else
+      {
+        ha_alter_info->report_unsupported_error("LOCK=NONE", "LOCK=SHARED");
+        return true;
+      }
     }
     return false;
   case HA_ALTER_ERROR:
@@ -254,6 +262,14 @@ Alter_info::algorithm(const THD *thd) const
   return requested_algorithm;
 }
 
+bool Alter_info::algorithm_is_nocopy(const THD *thd) const
+{
+  auto alg= algorithm(thd);
+  return alg == ALTER_TABLE_ALGORITHM_INPLACE
+         || alg == ALTER_TABLE_ALGORITHM_INSTANT
+         || alg == ALTER_TABLE_ALGORITHM_NOCOPY;
+}
+
 
 uint Alter_info::check_vcol_field(Item_field *item) const
 {
@@ -527,7 +543,7 @@ bool Sql_cmd_alter_table::execute(THD *thd)
     referenced from this structure will be modified.
     @todo move these into constructor...
   */
-  HA_CREATE_INFO create_info(lex->create_info);
+  Table_specification_st create_info(lex->create_info);
   Alter_info alter_info(lex->alter_info, thd->mem_root);
   create_info.alter_info= &alter_info;
   privilege_t priv(NO_ACL);
@@ -546,6 +562,8 @@ bool Sql_cmd_alter_table::execute(THD *thd)
     as for RENAME TO, as being done by SQLCOM_RENAME_TABLE
   */
   if ((alter_info.partition_flags & ALTER_PARTITION_DROP) ||
+      (alter_info.partition_flags & ALTER_PARTITION_CONVERT_IN) ||
+      (alter_info.partition_flags & ALTER_PARTITION_CONVERT_OUT) ||
       (alter_info.flags & ALTER_RENAME))
     priv_needed|= DROP_ACL;
 
@@ -563,6 +581,14 @@ bool Sql_cmd_alter_table::execute(THD *thd)
                    0, 0))
     DBUG_RETURN(TRUE);                  /* purecov: inspected */
 
+  if ((alter_info.partition_flags & ALTER_PARTITION_CONVERT_IN))
+  {
+    TABLE_LIST *tl= first_table->next_local;
+    tl->grant.privilege= first_table->grant.privilege;
+    tl->grant.m_internal= first_table->grant.m_internal;
+  }
+
+
   /* If it is a merge table, check privileges for merge children. */
   if (create_info.merge_list)
   {
