@@ -35,8 +35,7 @@
  *
  */
 
-#ifndef DDLPACKAGE_H
-#define DDLPACKAGE_H
+#pragma once
 
 #include <vector>
 #include <string>
@@ -47,11 +46,7 @@
 #include "bytestream.h"
 #include "logicalpartition.h"
 
-#if defined(_MSC_VER) && defined(xxxDDLPKG_DLLEXPORT)
-#define EXPORT __declspec(dllexport)
-#else
 #define EXPORT
-#endif
 
 namespace ddlpackage
 {
@@ -933,13 +928,13 @@ struct AtaRenameColumn : public AlterTableAction
 struct ColumnType
 {
   /** @brief Deserialize from ByteStream */
-  EXPORT virtual int unserialize(messageqcpp::ByteStream& bs);
+  EXPORT int unserialize(messageqcpp::ByteStream& bs);
 
   /** @brief Serialize to ByteStream */
-  EXPORT virtual int serialize(messageqcpp::ByteStream& bs);
+  EXPORT int serialize(messageqcpp::ByteStream& bs);
 
   /** @brief For deserialization. */
-  ColumnType() : fCharset(NULL), fExplicitLength(false)
+  ColumnType() : fCharset(NULL), fCollate(NULL), fCharsetNum(0), fExplicitLength(false)
   {
   }
 
@@ -983,6 +978,10 @@ struct ColumnType
 
   /** @brief Column charset (CHAR, VARCHAR and TEXT only) */
   const char* fCharset;
+  /** @brief Column collation (CHAR, VARCHAR and TEXT only) */
+  const char* fCollate;
+  /** @brief Column charset number (CHAR, VARCHAR and TEXT only) */
+  uint32_t fCharsetNum;
 
   /** @brief Is the TEXT column has explicit defined length, ie TEXT(1717) */
   bool fExplicitLength;
@@ -1570,5 +1569,3 @@ struct DropPartitionStatement : public SqlStatement
 }  // namespace ddlpackage
 
 #undef EXPORT
-
-#endif
