@@ -93,7 +93,8 @@ class DDLPackageProcessor
     NETWORK_ERROR,
     PARTITION_WARNING,
     WARN_NO_PARTITION,
-    DROP_TABLE_NOT_IN_CATALOG_ERROR
+    DROP_TABLE_NOT_IN_CATALOG_ERROR,
+    PP_LOST_CONNECTION
   };
 
   enum DebugLevel /** @brief Debug level type enumeration */
@@ -195,42 +196,6 @@ class DDLPackageProcessor
       int spare  : 6;
           Date( )   { year = 0; month = 0; day = 0; spare = 0;}
       }; */
-  /** @brief a structure to hold a datetime
-   */
-  struct dateTime
-  {
-    unsigned msecond : 20;
-    unsigned second : 6;
-    unsigned minute : 6;
-    unsigned hour : 6;
-    unsigned day : 6;
-    unsigned month : 4;
-    unsigned year : 16;
-    // NULL column value = 0xFFFFFFFFFFFFFFFE
-    EXPORT dateTime()
-    {
-      year = 0xFFFF;
-      month = 0xF;
-      day = 0x3F;
-      hour = 0x3F;
-      minute = 0x3F;
-      second = 0x3F;
-      msecond = 0xFFFFE;
-    }
-  };
-  /*
-      struct dateTime
-      {
-      int year    : 16;
-      int month   : 4;
-      int day     : 6;
-      int hour    : 6;
-      int minute  : 6;
-      int second  : 6;
-      int msecond : 20;
-          dateTime( )   { year = 0; month = 0; day = 0; hour = 0; minute = 0; second = 0; msecond = 0; }
-      }
-      ; */
   /** @brief a vector of dictionary object ids
    */
   typedef std::vector<DictOID> DictionaryOIDList;
@@ -283,6 +248,18 @@ class DDLPackageProcessor
     // std::cout << "in DDLPackageProcessor constructor " << this << std::endl;
   }
 
+  /** @brief Function wrapper for `processPackageInternal`.
+   */
+  DDLResult processPackage(ddlpackage::SqlStatement* sqlStmt);
+
+  /** @brief Check that give exception is related to PP lost connection.
+   */
+  bool checkPPLostConnection(std::string error);
+
+  /** @brief Internal implementation for `process` package command.
+   */
+  virtual DDLResult processPackageInternal(ddlpackage::SqlStatement* sqlStmt);
+
   /** @brief destructor
    */
   EXPORT virtual ~DDLPackageProcessor();
@@ -409,6 +386,8 @@ class DDLPackageProcessor
    */
   EXPORT void fetchLogFile(TableLogInfo& tableLogInfos, uint64_t uniqueId);
 
+  // virtual EXPORT DDLResult processPackage(ddlpackage::TruncTableStatement& truncTableStmt);
+
   BRM::TxnID fTxnid;
 
  protected:
@@ -871,6 +850,8 @@ class DDLPackageProcessor
   void cleanString(std::string& s);
   // std::string            fDDLLogFileName;
   DebugLevel fDebugLevel;  // internal use debug level
+
+  const std::string PPLostConnectionErrorCode = "MCS-2045";
 };
 /** @brief helper template function to do safe from string to type conversions
  *
@@ -885,4 +866,3 @@ bool from_string(T& t, const std::string& s, std::ios_base& (*f)(std::ios_base&)
 }  // namespace ddlpackageprocessor
 
 #undef EXPORT
-
